#ifndef NODE_HPP
#define NODE_HPP

#include <cstdlib>
#include <iostream>
#include <sstream>
#include <string>

using namespace std;

enum { NUM, VAR, ADD, SUB, MUL, DIV, POW, EXP, LOG, CHS };

class Node {
	public:
		virtual int type(void) const = 0;
		virtual double eval( double ) const = 0;
		virtual string write(void) const = 0;
		virtual Node* deriv(void) const = 0;
		virtual Node* copy(void) const = 0;
};

class Const : public Node {
	private:
		double val;
	public:
		Const(double);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Var : public Node {
	public:
		Var(void);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Func : public Node {
	protected:
		Node* rgt;
	public:
		Func(Node*);
		virtual int type(void) const = 0;
		virtual double eval( double ) const = 0;
		virtual string write(void) const = 0;
		virtual Node* deriv(void) const = 0;
		virtual Node* copy(void) const = 0;
};

class Chs : public Func {
	public:
		Chs(Node*);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class OpBin : public Node {
	protected:
		Node* lft;
		Node* rgt;
	public:
		OpBin(Node*, Node*);
		virtual int type(void) const = 0;
		virtual double eval( double ) const = 0;
		virtual string write(void) const = 0;
		virtual Node* deriv(void) const = 0;
		virtual Node* copy(void) const = 0;
};

class Add : public OpBin {
	public:
		Add(Node *, Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Sub : public OpBin {
	public:
		Sub(Node *, Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Mul : public OpBin {
	public:
		Mul(Node *, Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Div : public OpBin {
	public:
		Div(Node *, Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Pow : public OpBin {
	public:
		Pow(Node *, Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Log : public Func {
	public:
		Log(Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

class Exp : public Func {
	public:
		Exp(Node *);
		virtual int type(void) const;
		virtual double eval( double ) const;
		virtual string write(void) const;
		virtual Node* deriv(void) const;
		virtual Node* copy(void) const;
};

#endif






